@@ -203,6 +203,8 @@ build_env(Arg, Scriptfilename, Pathinfo, ExtraEnv, SC) ->
     case H#headers.authorization of
         undefined ->
             AuthEnv = [];
+        {undefined, _, _} ->
+            AuthEnv = [];
         {User, Password, "Basic " ++ Auth64} ->
             AuthEnv = [
                        {"HTTP_AUTHORIZATION", "Basic " ++ Auth64},
@@ -226,6 +228,25 @@ build_env(Arg, Scriptfilename, Pathinfo, ExtraEnv, SC) ->
 				   end,
 				   SC#sconf.extra_cgi_vars),
 
+    %% Some versions of erlang:open_port can't handle query strings that
+    %% end with an equal sign. This is because the broken versions treat
+    %% environment variable strings ending with '=' as environment variable
+    %% names intended to be deleted from the environment, i.e. as if they
+    %% have no value. The result is that no QUERY_STRING environment
+    %% variable gets set for these cases. We work around this by appending
+    %% a & character to any query string that ends in =.
+    QueryString = case checkdef(Arg#arg.querydata) of
+                      "" ->
+                          "";
+                      QS ->
+                          case lists:reverse(QS) of
+                              [$= | _] ->
+                                  QS ++ "&";
+                              _ ->
+                                  QS
+                          end
+                  end,
+
     %%todo - review. should AuthEnv entries be overridable by ExtraEnv or not?
     %% we should define policy here rather than let through dupes.
 
@@ -247,7 +268,7 @@ build_env(Arg, Scriptfilename, Pathinfo, ExtraEnv, SC) ->
           ([
             {"SERVER_SOFTWARE", "Yaws/"++yaws_generated:version()},
             {"SERVER_NAME", Hostname},
-            {"HTTP_HOST", Hostname},
+            {"HTTP_HOST", checkdef(H#headers.host)},
             {"GATEWAY_INTERFACE", "CGI/1.1"},
             {"SERVER_PROTOCOL", "HTTP/" ++ integer_to_list(Maj) ++
              "." ++ integer_to_list(Min)},
@@ -256,10 +277,11 @@ build_env(Arg, Scriptfilename, Pathinfo, ExtraEnv, SC) ->
             {"REQUEST_URI", RequestURI},
             {"DOCUMENT_ROOT",         Arg#arg.docroot},
             {"DOCUMENT_ROOT_MOUNT", Arg#arg.docroot_mount},
-            {"SCRIPT_FILENAME", Scriptfilename},% For PHP 4.3.2 and higher
-                                                % see http://bugs.php.net/bug.php?id=28227
-                                                % (Sergei Golovan).
-                                                % {"SCRIPT_TRANSLATED", Scriptfilename},   %IIS6+
+            %% SCRIPT_FILENAME is for PHP 4.3.2 and higher
+            %% see http://bugs.php.net/bug.php?id=28227
+            %% (Sergei Golovan).
+            {"SCRIPT_FILENAME", Scriptfilename},
+            %% {"SCRIPT_TRANSLATED", Scriptfilename},   %IIS6+
             {"PATH_INFO",                Pathinfo2},
             {"PATH_TRANSLATED",        PathTranslated},
             %% <JMN_2007-02>
@@ -293,7 +315,7 @@ build_env(Arg, Scriptfilename, Pathinfo, ExtraEnv, SC) ->
             %% CGI/1.1 spec to substitute REMOTE_ADDR
             {"SERVER_ADDR", LocalAddr},   %% Apache compat
             {"LOCAL_ADDR", LocalAddr},    %% IIS compat
-            {"QUERY_STRING", checkdef(Arg#arg.querydata)},
+            {"QUERY_STRING", QueryString},
             {"CONTENT_TYPE", H#headers.content_type},
             {"CONTENT_LENGTH", H#headers.content_length},
             {"HTTP_ACCEPT", H#headers.accept},
@@ -597,29 +619,32 @@ cgi_add_resp(Bin, Port) ->
 -define(FCGI_VERSION_1, 1).
 
 -define(FCGI_TYPE_BEGIN_REQUEST, 1).
--define(FCGI_TYPE_ABORT_REQUEST, 2).
+%%% Not needed yet
+%%%-define(FCGI_TYPE_ABORT_REQUEST, 2).
 -define(FCGI_TYPE_END_REQUEST, 3).
 -define(FCGI_TYPE_PARAMS, 4).
 -define(FCGI_TYPE_STDIN, 5).
 -define(FCGI_TYPE_STDOUT, 6).
 -define(FCGI_TYPE_STDERR, 7).
--define(FCGI_TYPE_DATA, 8).
--define(FCGI_TYPE_GET_VALUES, 9).
--define(FCGI_TYPE_GET_VALUES_RESULT, 10).
+%%% Not needed yet
+%%%-define(FCGI_TYPE_DATA, 8).
+%%%-define(FCGI_TYPE_GET_VALUES, 9).
+%%%-define(FCGI_TYPE_GET_VALUES_RESULT, 10).
 -define(FCGI_TYPE_UNKNOWN_TYPE, 11).
 
 fcgi_type_name(?FCGI_TYPE_BEGIN_REQUEST) -> "begin-request";
-fcgi_type_name(?FCGI_TYPE_ABORT_REQUEST) -> "abort-request";
+%%% Not needed yet
+%%%fcgi_type_name(?FCGI_TYPE_ABORT_REQUEST) -> "abort-request";
 fcgi_type_name(?FCGI_TYPE_END_REQUEST) -> "end-request";
 fcgi_type_name(?FCGI_TYPE_PARAMS) -> "params";
 fcgi_type_name(?FCGI_TYPE_STDIN) -> "stdin";
 fcgi_type_name(?FCGI_TYPE_STDOUT) -> "stdout";
 fcgi_type_name(?FCGI_TYPE_STDERR) -> "stderr";
-fcgi_type_name(?FCGI_TYPE_DATA) -> "data";
-fcgi_type_name(?FCGI_TYPE_GET_VALUES) -> "get_values";
-fcgi_type_name(?FCGI_TYPE_GET_VALUES_RESULT) -> "get_values_result";
-fcgi_type_name(?FCGI_TYPE_UNKNOWN_TYPE) -> "unknown-type";
-fcgi_type_name(_) -> "?".
+%%% Not needed yet
+%%%fcgi_type_name(?FCGI_TYPE_DATA) -> "data";
+%%%fcgi_type_name(?FCGI_TYPE_GET_VALUES) -> "get_values";
+%%%fcgi_type_name(?FCGI_TYPE_GET_VALUES_RESULT) -> "get_values_result";
+fcgi_type_name(?FCGI_TYPE_UNKNOWN_TYPE) -> "unknown-type".
 
 %%% The FCGI implementation does not support handling concurrent requests
 %%% over a connection; it creates a separate connection for each
@@ -752,7 +777,8 @@ fcgi_worker_fail(WorkerState, Reason) ->
     ParentPid = WorkerState#fcgi_worker_state.parent_pid,
     ParentPid ! {self(), failure, Reason},
     error_logger:error_msg("FastCGI failure: ~p~n", [Reason]),
-    exit(Reason).
+    %% exit normally to avoid filling log with crash messages
+    exit(normal).
 
 fcgi_worker_fail_if(true, WorkerState, Reason) ->
     fcgi_worker_fail(WorkerState, Reason);
@@ -765,10 +791,15 @@ fcgi_start_worker(Role, Arg, ServerConf, Options) ->
 
 
 fcgi_worker(ParentPid, Role, Arg, ServerConf, Options) ->
-    AppServerHost = get_opt(app_server_host, Options,
-                            ServerConf#sconf.fcgi_app_server_host),
-    AppServerPort = get_opt(app_server_port, Options,
-                            ServerConf#sconf.fcgi_app_server_port),
+    {DefaultSvrHost, DefaultSvrPort} =
+        case ServerConf#sconf.fcgi_app_server of
+            undefined ->
+                {undefined, undefined};
+            Else ->
+                Else
+        end,
+    AppServerHost = get_opt(app_server_host, Options, DefaultSvrHost),
+    AppServerPort = get_opt(app_server_port, Options, DefaultSvrPort),
     PreliminaryWorkerState = #fcgi_worker_state{parent_pid = ParentPid},
     fcgi_worker_fail_if(AppServerHost == undefined, PreliminaryWorkerState,
                         "app server host must be configured"),
@@ -801,19 +832,19 @@ fcgi_worker(ParentPid, Role, Arg, ServerConf, Options) ->
             TraceProtocol,
             LogAppError]),
     WorkerState = #fcgi_worker_state{
-                app_server_host = AppServerHost,
-                app_server_port = AppServerPort,
-                path_info = PathInfo,
-                env = Env,
-                keep_connection = false,        % Currently hard-coded; make
-                                                % configurable in the future?
-                trace_protocol = TraceProtocol,
-                log_app_error = LogAppError,
-                role = Role,
-                parent_pid = ParentPid,
-                yaws_worker_pid = Arg#arg.pid,
-                app_server_socket = AppServerSocket
-            },
+      app_server_host = AppServerHost,
+      app_server_port = AppServerPort,
+      path_info = PathInfo,
+      env = Env,
+      keep_connection = false,                % Currently hard-coded; make
+                                              % configurable in the future?
+      trace_protocol = TraceProtocol,
+      log_app_error = LogAppError,
+      role = Role,
+      parent_pid = ParentPid,
+      yaws_worker_pid = Arg#arg.pid,
+      app_server_socket = AppServerSocket
+     },
     fcgi_send_begin_request(WorkerState),
     fcgi_send_params(WorkerState, Env),
     fcgi_send_params(WorkerState, []),
@@ -847,11 +878,13 @@ fcgi_connect_to_application_server(WorkerState, Host, Port) ->
 
 
 fcgi_send_begin_request(WorkerState) ->
-    KeepConnection = WorkerState#fcgi_worker_state.keep_connection,
-    Flags = case KeepConnection of
-        true -> ?FCGI_KEEP_CONN;
-        false -> ?FCGI_DONT_KEEP_CONN
-    end,
+    %% Not needed yet -- keep_connection is currently hard-coded to false
+    %%KeepConnection = WorkerState#fcgi_worker_state.keep_connection,
+    %%Flags = case KeepConnection of
+    %%            true -> ?FCGI_KEEP_CONN;
+    %%            false -> ?FCGI_DONT_KEEP_CONN
+    %%        end,
+    Flags = ?FCGI_DONT_KEEP_CONN,
     Role = WorkerState#fcgi_worker_state.role,
     fcgi_send_record(WorkerState, ?FCGI_TYPE_BEGIN_REQUEST,
                      ?FCGI_REQUEST_ID_APPLICATION, <<Role:16, Flags:8, 0:40>>).
@@ -996,7 +1029,7 @@ fcgi_encode_name_value_list(_NameValueList = [{Name, Value} | Tail]) ->
 fcgi_encode_name_value(Name, _Value = undefined) ->
     fcgi_encode_name_value(Name, "");
 fcgi_encode_name_value(Name, Value) when is_list(Name) and is_list(Value) ->
-    NameSize = length(Name),
+    NameSize = iolist_size(Name),
     %% If name size is < 128, encode it as one byte with the high bit clear.
     %% If the name size >= 128, encoded it as 4 bytes with the high bit set.
     NameSizeData = if
@@ -1004,19 +1037,16 @@ fcgi_encode_name_value(Name, Value) when is_list(Name) and is_list(Value) ->
                            <<NameSize:8>>;
                        true ->
                            <<(NameSize bor 16#80000000):32>>
-                               end,
+                   end,
     %% Same encoding for the value size.
-    ValueSize = length(Value),
+    ValueSize = iolist_size(Value),
     ValueSizeData = if
-        ValueSize < 128 ->
-            <<ValueSize:8>>;
-        true ->
-            <<(ValueSize bor 16#80000000):32>>
-    end,
-    <<NameSizeData/binary,
-      ValueSizeData/binary,
-      (list_to_binary(Name))/binary,
-      (list_to_binary(Value))/binary>>.
+                        ValueSize < 128 ->
+                            <<ValueSize:8>>;
+                        true ->
+                            <<(ValueSize bor 16#80000000):32>>
+                    end,
+    list_to_binary([<<NameSizeData/binary, ValueSizeData/binary>>, Name, Value]).
 
 
 fcgi_header_loop(WorkerState) ->
@@ -1031,12 +1061,18 @@ fcgi_header_loop(WorkerState, LineState) ->
         {_EmptyLine = [], NewLineState} ->
             case NewLineState of
                 {middle, Data} ->
-                    ParentPid ! {self(), partial_data, Data},
-                    receive
-                        {ParentPid, stream_data} ->
-                            fcgi_data_loop(WorkerState);
-                        {ParentPid, no_data} ->
-                            ok
+                    case WorkerState#fcgi_worker_state.role of
+                        ?FCGI_ROLE_AUTHORIZER ->
+                            % For authorization we never stream to the client
+                            fcgi_collect_all_data_loop(WorkerState, Data);
+                        _ ->
+                            ParentPid ! {self(), partial_data, Data},
+                            receive
+                                {ParentPid, stream_data} ->
+                                    fcgi_stream_data_loop(WorkerState);
+                                {ParentPid, no_data} ->
+                                    ok
+                            end
                     end;
                 {ending, Data} ->
                     ParentPid ! {self(), all_data, Data},
@@ -1091,17 +1127,35 @@ fcgi_add_resp(WorkerState, OldData) ->
     end.
 
 
-fcgi_data_loop(WorkerState) ->
+fcgi_stream_data_loop(WorkerState) ->
     YawsWorkerPid = WorkerState#fcgi_worker_state.yaws_worker_pid,
     case fcgi_get_output(WorkerState) of
         {data, Data} ->
             yaws_api:stream_chunk_deliver_blocking(YawsWorkerPid, Data),
-            fcgi_data_loop(WorkerState);
+            fcgi_stream_data_loop(WorkerState);
         {exit_status, _Status} ->
             yaws_api:stream_chunk_end(YawsWorkerPid)
     end.
 
 
+fcgi_collect_all_data_loop(WorkerState, Data) ->
+    YawsWorkerPid = WorkerState#fcgi_worker_state.yaws_worker_pid,
+    case fcgi_get_output(WorkerState) of
+        {data, MoreData} ->
+            NewData = <<Data/binary, MoreData/binary>>,
+            fcgi_collect_all_data_loop(WorkerState, NewData);
+        {exit_status, _Status} ->
+            ParentPid = WorkerState#fcgi_worker_state.parent_pid,
+            ParentPid ! {self(), all_data, Data},
+            receive
+                {ParentPid, stream_data} ->
+                    yaws_api:stream_chunk_end(YawsWorkerPid);
+                {ParentPid, no_data} ->
+                    ok
+            end
+    end.
+
+
 fcgi_get_output(WorkerState) ->
     {Type, ContentData} = fcgi_receive_record(WorkerState),
     case Type of
@@ -1151,15 +1205,12 @@ fcgi_get_output(WorkerState) ->
             fcgi_worker_fail(
               WorkerState,
               {"application did not understand record type we sent",
-               UnknownType});
-        OtherType ->
-            fcgi_worker_fail(WorkerState,
-                             {"received unknown record type", OtherType})
+               UnknownType})
     end.
 
 
 fcgi_receive_record(WorkerState) ->
-    {ok, Header} = fcgi_receive_binary(WorkerState, 8, ?FCGI_READ_TIMEOUT_MSECS),
+    Header = fcgi_receive_binary(WorkerState, 8, ?FCGI_READ_TIMEOUT_MSECS),
     <<Version:8, Type:8, RequestId:16, ContentLength:16,
       PaddingLength:8, Reserved:8>> = Header,
     fcgi_worker_fail_if(Version /= 1, WorkerState,
@@ -1194,42 +1245,30 @@ fcgi_receive_record(WorkerState) ->
         OtherType ->
             throw({"received unexpected type", OtherType})
     end,
-    Content = case ContentLength of
-                  0 ->
-                      {ok, <<>>};
-                  _ ->
-                      fcgi_receive_binary(WorkerState, ContentLength,
-                                          ?FCGI_READ_TIMEOUT_MSECS)
-              end,
-    case Content of
-        {error, Reason} ->
-            fcgi_worker_fail(WorkerState,
-                             {"unable to read content data", Reason});
-        {ok, ContentData} ->
-            case PaddingLength of
-                0 ->
-                    {Type, ContentData};
-                _ ->
-                    case fcgi_receive_binary(WorkerState, PaddingLength,
-                                             ?FCGI_READ_TIMEOUT_MSECS) of
-                        {error, Reason} ->
-                            fcgi_worker_fail(
-                              WorkerState,
-                              {"unable to read record padding data", Reason});
-                        {ok, PaddingData} ->
-                            fcgi_trace_protocol(WorkerState, "Receive",
-                                                Version, Type, RequestId,
-                                                ContentLength, PaddingLength,
-                                                Reserved, ContentData,
-                                                PaddingData),
-                            {Type, ContentData}
-                    end
-            end
+    ContentData = case ContentLength of
+                      0 ->
+                          <<>>;
+                      _ ->
+                          fcgi_receive_binary(WorkerState, ContentLength,
+                                              ?FCGI_READ_TIMEOUT_MSECS)
+                  end,
+    case PaddingLength of
+        0 ->
+            {Type, ContentData};
+        _ ->
+            PaddingData = fcgi_receive_binary(WorkerState, PaddingLength,
+                                              ?FCGI_READ_TIMEOUT_MSECS),
+            fcgi_trace_protocol(WorkerState, "Receive",
+                                Version, Type, RequestId,
+                                ContentLength, PaddingLength,
+                                Reserved, ContentData,
+                                PaddingData),
+            {Type, ContentData}
     end.
 
 
 fcgi_receive_binary(_WorkerState, Length, _Timeout) when Length == 0 ->
-    {ok, <<>>};
+    <<>>;
 fcgi_receive_binary(WorkerState, Length, Timeout) ->
     AppServerSocket = WorkerState#fcgi_worker_state.app_server_socket,
     case gen_tcp:recv(AppServerSocket, Length, Timeout) of
@@ -1237,7 +1276,7 @@ fcgi_receive_binary(WorkerState, Length, Timeout) ->
             fcgi_worker_fail(WorkerState,
                              {"recv from application server failed", Reason});
         {ok, Data} ->
-            {ok, Data}
+            Data
     end.
 
 
