@@ -1,7 +1,7 @@
 %%----------------------------------------------------------------------
 %%% File    : yaws_log.erl
 %%% Author  : Claes Wikstrom <klacke@hyber.org>
-%%% Purpose : 
+%%% Purpose :
 %%% Created : 26 Jan 2002 by Claes Wikstrom <klacke@hyber.org>
 %%%----------------------------------------------------------------------
 
@@ -16,16 +16,27 @@
 -export([start_link/0]).
 
 %% gen_server callbacks
--export([init/1, handle_call/3, handle_cast/2, handle_info/2, terminate/2, 
-         handle_event/2]).
--export([accesslog/5,
-         accesslog/7,
-         accesslog/8,
-         setdir/2,
+-export([init/1, handle_call/3, handle_cast/2, handle_info/2, terminate/2,
+         code_change/3]).
+
+%% API
+-export([accesslog/6,
+         setup/2,
          open_trace/1,
          trace_traffic/2,
+         trace_tty/1,
          authlog/4,
-         actl_trace/1]).
+         rotate/1,
+         add_sconf/1,
+         del_sconf/1]).
+
+%% yaws_logger callbacks
+-export([
+         open_log/3,
+         close_log/3,
+         wrap_log/4,
+         write_log/4
+        ]).
 
 
 -include("../include/yaws.hrl").
@@ -43,14 +54,7 @@
           tracefd,
           log_wrap_size = ?WRAP_LOG_SIZE,
           tty_trace = false,
-          copy_errlog,
-          auth_log,
-          alogs =  []}).
-
-
--record(alog, {fd,
-               servername,
-               filename}).
+          copy_errlog}).
 
 
 %%%----------------------------------------------------------------------
@@ -59,32 +63,86 @@
 start_link() ->
     gen_server:start_link({local, yaws_log}, yaws_log, [], []).
 
-accesslog(ServerName, Ip, Req, Status, Length) ->
-    accesslog(ServerName, Ip, "-", Req, Status, Length, "-", "-").
+setup(GC, Sconfs) ->
+    gen_server:call(?MODULE, {setup, GC, Sconfs}, infinity).
 
-accesslog(ServerName, Ip, Req, Status, Length, Referrer, UserAgent) ->
-    accesslog(ServerName, Ip, "-", Req, Status, Length, Referrer, UserAgent).
+add_sconf(SConf) ->
+    gen_server:call(yaws_log, {soft_add_sc, SConf}, infinity).
 
-accesslog(ServerName, Ip, User, Req, Status, Length, Referrer, UserAgent) ->
-    gen_server:cast(?MODULE, {access, ServerName, Ip, User, Req, 
-                              Status, Length, Referrer, UserAgent}).
-setdir(GC, Sconfs) ->
-    gen_server:call(?MODULE, {setdir, GC, Sconfs}, infinity).
+del_sconf(SConf) ->
+    gen_server:call(yaws_log, {soft_del_sc, SConf}, infinity).
+
+accesslog(SConf, Ip, Req, InH, OutH, Time) ->
+    catch yaws_logger:accesslog(SConf, Ip, Req, InH, OutH, Time).
+
+authlog(SConf, IP, Path, Item) ->
+    catch yaws_logger:authlog(SConf, IP, Path, Item).
+
+rotate(Res) ->
+    gen_server:cast(?MODULE, {yaws_hupped, Res}).
 
 open_trace(What) ->
     gen_server:call(?MODULE, {open_trace, What}, infinity).
+
 trace_traffic(ServerOrClient , Data) ->
     gen_server:cast(?MODULE, {trace, ServerOrClient, Data}).
 
-authlog(ServerName, IP, Path, Item) ->
-    gen_server:cast(?MODULE, {auth, ServerName, IP, Path, Item}).
+trace_tty(Bool) ->
+    gen_server:call(?MODULE, {trace_tty, Bool}, infinity).
 
+%% XXX: Dead code
 %% from external ctl prog
-actl_trace(What) ->
-    gen_server:call(?MODULE, {actl_trace, What}, infinity).
+%%actl_trace(What) ->
+%%    gen_server:call(?MODULE, {actl_trace, What}, infinity).
 
 
+%%%----------------------------------------------------------------------
+%%% Callback functions from yaws_logger
+%%%----------------------------------------------------------------------
+open_log(ServerName, Type, Dir) ->
+    FileName = case os:type() of
+                   {win32,_ } ->
+                       lists:map(fun($:) -> $.;
+                                    (C ) -> C
+                                 end, ServerName);
+                   _ ->
+                       ServerName
+               end,
+    A = filename:join([Dir, FileName ++ "." ++ atom_to_list(Type)]),
+    case file:open(A, [write, raw, append]) of
+        {ok, Fd} ->
+            {true, {Fd, A}};
+        _Err ->
+            error_logger:format("Cannot open ~p",[A]),
+            false
+    end.
 
+close_log(_ServerName, _Type, {Fd, _FileName}) ->
+    file:close(Fd).
+
+wrap_log(_ServerName, _Type, {Fd, FileName}, LogWrapSize) ->
+    case wrap_p(FileName, LogWrapSize) of
+        true ->
+            file:close(Fd),
+            Old = [FileName, ".old"],
+            file:delete(Old),
+            file:rename(FileName, Old),
+            {ok, Fd2} = file:open(FileName, [write, raw]),
+            error_logger:info_msg("Wrap log ~p",[FileName]),
+            {Fd2, FileName};
+        false ->
+            {Fd, FileName};
+        enoent ->
+            %% Logfile disappeared,
+            error_logger:format("Logfile ~p disappeared - we reopen it",
+                                [FileName]),
+            file:close(Fd),
+            {ok, Fd2} = file:open(FileName, [write, raw]),
+            {Fd2, FileName}
+    end.
+
+write_log(ServerName, Type, {Fd, _FileName}, Infos) ->
+    gen_server:cast(yaws_log, {ServerName, Type, Fd, Infos}).
 
 %%%----------------------------------------------------------------------
 %%% Callback functions from gen_server
@@ -99,6 +157,7 @@ actl_trace(What) ->
 %%----------------------------------------------------------------------
 init([]) ->
     process_flag(trap_exit, true),
+    ets:new(yaws_log, [named_table, set, protected, {keypos, 2}]),
     {ok, #state{running = false, now = fmtnow()}}.
 
 %%----------------------------------------------------------------------
@@ -110,67 +169,29 @@ init([]) ->
 %%          {stop, Reason, Reply, State}   | (terminate/2 is called)
 %%          {stop, Reason, State}            (terminate/2 is called)
 %%----------------------------------------------------------------------
-handle_call({setdir, GC, Sconfs}, _From, State) 
+handle_call({setup, GC, Sconfs}, _From, State)
   when State#state.running == false ->
     Dir = GC#gconf.logdir,
-    ?Debug("setdir ~s~n", [Dir]),
+    ?Debug("setup ~s~n", [Dir]),
     ElogFile = filename:join([Dir, "report.log"]),
     Copy = if ?gc_has_copy_errlog(GC) ->
-                   gen_event:add_handler(error_logger, yaws_log_file_h, 
+                   gen_event:add_handler(error_logger, yaws_log_file_h,
                                          ElogFile),
                    true;
               true ->
                    false
            end,
     SCs = lists:flatten(Sconfs),
-    L = lists:zf(
-          fun(SC) when ?sc_has_access_log(SC)  ->
-                  FileName = case os:type() of
-                                 {win32,_ } ->
-                                     lists:map(fun($:) -> $.;
-                                                              (C ) -> C
-                                                       end,
-                                               SC#sconf.servername);
-                                 _ ->
-                                     SC#sconf.servername
-                             end,
-                  A = filename:join([Dir, FileName ++ ".access"]),
-                  case file:open(A, [write, raw, append]) of
-                      {ok, Fd} ->
-                          {true, #alog{servername = SC#sconf.servername,
-                                       fd = Fd,
-                                       filename = A}};
-                      _Err ->
-                          error_logger:format("Cannot open ~p~n",[A]),
-                          false
-                  end;
-             (_) ->
-                  false
-          end, SCs),
+    lists:foreach(fun(SC) ->
+                          yaws_logger:open_log(SC, auth, Dir),
+                          yaws_logger:open_log(SC, access, Dir)
+                  end, SCs),
 
-    AuthLogFileName = filename:join([Dir, "auth.log"]),
-    AuthLog = 
-        if ?gc_has_auth_log(GC) ->
-                case file:open(AuthLogFileName, [write, raw, append]) of
-                    {ok, AFd} ->
-                        #alog{fd = AFd,
-                              servername = undefined,
-                              filename = AuthLogFileName};
-                    _Err ->
-                        error_logger:format("Cannot open ~s~n", 
-                                            [AuthLogFileName]),
-                        undefined
-                end;
-           true ->
-                undefined
-        end,
     S2 = State#state{running = true,
                      dir  = Dir,
                      now = fmtnow(),
                      log_wrap_size = GC#gconf.log_wrap_size,
-                     copy_errlog = Copy,
-                     auth_log = AuthLog,
-                     alogs = L},
+                     copy_errlog = Copy},
 
     yaws:ticker(3000, secs3),
     yaws:ticker(10 * 60 * 1000, minute10),
@@ -182,19 +203,19 @@ handle_call({setdir, GC, Sconfs}, _From, State)
 %% We can't ever change logdir, we can however
 %% change logging opts for various servers
 
-handle_call({setdir, GC, Sconfs}, _From, State) 
+handle_call({setup, GC, Sconfs}, _From, State)
   when State#state.running == true ->
 
     Dir = State#state.dir,
     ElogFile = filename:join([Dir, "report.log"]),
     Copy = if ?gc_has_copy_errlog(GC), State#state.copy_errlog == false->
-                   gen_event:add_handler(error_logger, yaws_log_file_h, 
+                   gen_event:add_handler(error_logger, yaws_log_file_h,
                                          ElogFile),
                    true;
               ?gc_has_copy_errlog(GC) ->
                    true;
               State#state.copy_errlog == true ->
-                   gen_event:delete_handler(error_logger, yaws_log_file_h, 
+                   gen_event:delete_handler(error_logger, yaws_log_file_h,
                                             normal),
                    false;
               true ->
@@ -202,43 +223,36 @@ handle_call({setdir, GC, Sconfs}, _From, State)
            end,
 
     %% close all files
-    lists:map(
-      fun(AL) ->
-              file:close(AL#alog.fd)
-      end, State#state.alogs),
-    SCs = lists:flatten(Sconfs),
+    yaws_logger:close_logs(),
 
     %% reopen logfiles
-    L = lists:zf(fun(SC) -> open_alog(Dir, SC) end,
-                 SCs),
+    SCs = lists:flatten(Sconfs),
+    lists:foreach(fun(SC) ->
+                          yaws_logger:open_log(SC, auth, Dir),
+                          yaws_logger:open_log(SC, access, Dir)
+                  end, SCs),
+
     S2 = State#state{running = true,
                      dir  = Dir,
                      now = fmtnow(),
-                     copy_errlog = Copy,
-                     alogs = L},
+                     copy_errlog = Copy},
 
     {reply, ok, S2};
 
 
 %% a virt server has been added
 handle_call({soft_add_sc, SC}, _From, State) ->
-    case open_alog(State#state.dir, SC) of
-        {true, A} ->
-            {reply, ok, State#state{alogs = [A|State#state.alogs]}};
-        false ->
-            {reply, ok, State}
-    end;
+    yaws_logger:open_log(SC, auth, State#state.dir),
+    yaws_logger:open_log(SC, access, State#state.dir),
+    {reply, ok, State};
 
 %% a virt server has been deleted
 handle_call({soft_del_sc, SC}, _From, State) ->
-    case lists:keysearch(SC#sconf.servername, #alog.servername,
-                         State#state.alogs) of
-        {value, A} ->
-            file:close(A#alog.fd),
-            {reply,ok, State#state{alogs = State#state.alogs -- [A]}};
-        false ->
-            {reply,ok,State}
-    end;
+    %%yaws_logger:close_log(SC#sconf.servername, auth),
+    %%yaws_logger:close_log(SC#sconf.servername, access),
+    yaws_logger:close_log(SC, auth),
+    yaws_logger:close_log(SC, access),
+    {reply, ok, State};
 
 
 handle_call({open_trace, What}, _From, State) ->
@@ -269,21 +283,67 @@ handle_call(state, _From, State) ->
 %%          {noreply, State, Timeout} |
 %%          {stop, Reason, State}            (terminate/2 is called)
 %%----------------------------------------------------------------------
-handle_cast({access, ServerName, Ip, User, Req, Status, 
-             Length, Referrer, UserAgent}, State) ->
-    case State#state.running of 
+handle_cast({_ServerName, access, Fd, {Ip, Req, InH, OutH, _}}, State) ->
+    case State#state.running of
         true ->
-            do_alog(ServerName, Ip, User, Req, Status, Length, 
-                    Referrer, UserAgent, State),
+            Status = case OutH#outh.status of
+                         undefined -> "-";
+                         S         -> integer_to_list(S)
+                     end,
+            Len = case Req#http_request.method of
+                      'HEAD' ->
+                          "-";
+                      _ ->
+                          case OutH#outh.contlen of
+                              undefined ->
+                                  case OutH#outh.act_contlen of
+                                      undefined -> "-";
+                                      L         -> integer_to_list(L)
+                                  end;
+                              L ->
+                                  integer_to_list(L)
+                          end
+                  end,
+            Ver = case Req#http_request.version of
+                      {1,0} -> "HTTP/1.0";
+                      {1,1} -> "HTTP/1.1";
+                      {0,9} -> "HTTP/0.9"
+                  end,
+
+            Path      = safe_decode_path(Req#http_request.path),
+            Meth      = yaws:to_list(Req#http_request.method),
+            Referer   = optional_header(InH#headers.referer),
+            UserAgent = optional_header(InH#headers.user_agent),
+            User      = case InH#headers.authorization of
+                            {U, _P, _OStr} -> U;
+                            _              -> "-"
+                        end,
+
+            Msg = fmt_access_log(State#state.now, Ip, User,
+                                 [Meth, $\s, Path, $\s, Ver],
+                                 Status,  Len, Referer, UserAgent),
+            file:write(Fd, Msg),
             {noreply, State};
         false ->
             {noreply, State}
     end;
 
-handle_cast({auth, ServerName, IP, Path, Item}, State) ->
-    case State#state.running of 
-        true when State#state.auth_log /= undefined ->
-            do_auth_log(ServerName, IP, Path, Item, State),
+handle_cast({ServerName, auth, Fd, {Ip, Path, Item}}, State) ->
+    case State#state.running of
+        true ->
+            IpStr = case catch inet_parse:ntoa(Ip) of
+                        {'EXIT', _} -> "unknownip";
+                        Val -> Val
+                    end,
+            Msg = [IpStr, " ", State#state.now, " ", ServerName, " " ,
+                   "\"", Path,"\"",
+                   case Item of
+                       {ok, User}       -> [" OK user=", User];
+                       403              -> [" 403"];
+                       {401, Realm}     -> [" 401 realm=", Realm];
+                       {401, User, PWD} -> [" 401 user=", User, " badpwd=", PWD]
+                   end, "\n"],
+            file:write(Fd, Msg),
             {noreply, State};
         false ->
             {noreply,State}
@@ -298,57 +358,10 @@ handle_cast({trace, from_client, Data}, State) ->
     Str = ["*** CLI -> SRV *** ", Data],
     file:write(State#state.tracefd, Str),
     tty_trace(Str, State),
-    {noreply, State}.
-
-
-open_alog(Dir, SC) when ?sc_has_access_log(SC) ->
-    A = filename:join([Dir, SC#sconf.servername ++ ".access"]),
-    case file:open(A, [write, raw, append]) of
-        {ok, Fd} ->
-            {true, #alog{servername = SC#sconf.servername,
-                         fd = Fd,
-                         filename = A}};
-        _Err ->
-            error_logger:format("Cannot open ~p",[A]),
-            false
-    end;
-open_alog(_,_) ->
-    false.
-
-
-
-do_alog(ServerName, Ip, User, Req, Status, Length, Referrer,
-        UserAgent, State) ->
-    case lists:keysearch(ServerName, #alog.servername, State#state.alogs) of
-        {value, AL} ->
-            I = fmt_alog(State#state.now, Ip, User, Req, Status,  Length, 
-                         Referrer, UserAgent),
-            file:write(AL#alog.fd, I);
-        _ ->
-            false
-    end.
-
-do_auth_log(ServerName, IP, Path, Item, State) ->
-    AL = State#state.auth_log,
-    IpStr = case catch inet_parse:ntoa(IP) of
-                {'EXIT', _} -> "unknownip";
-                Val -> Val
-            end,
-    I = [IpStr,
-         " ", State#state.now,
-         " ", ServerName, " " ,"\"", Path,"\"",
-         case Item of
-             {ok, User} ->
-                 [" OK user=", User];
-             403 ->
-                 [" 403"];
-             {401, Realm} ->
-                 [" 401 realm=", Realm];
-             {401, User, PWD} ->
-                 [" 401 user=", User, " badpwd=", PWD]
-         end, "\n"],
-    file:write(AL#alog.fd, I).
+    {noreply, State};
 
+handle_cast({yaws_hupped, _}, State) ->
+    handle_info(minute10, State).
 
 
 tty_trace(Str, State) ->
@@ -359,16 +372,18 @@ tty_trace(Str, State) ->
             io:format("~s", [binary_to_list(list_to_binary([Str]))])
     end.
 
+%% XXX: yaws_log is not a gen_event and is not an event handler on
+%% yaws_event_manager.
 %%----------------------------------------------------------------------
 %% Func: handle_event/2
-%% Returns: 
+%% Returns:
 %%----------------------------------------------------------------------
-handle_event({yaws_hupped, _Res}, State) ->
-    handle_info(minute10,State),
-    {ok, State};
+%%handle_event({yaws_hupped, _Res}, State) ->
+%%    handle_info(minute10,State),
+%%    {ok, State};
 
-handle_event(_Event, State) ->
-    {ok, State}.
+%%handle_event(_Event, State) ->
+%%    {ok, State}.
 
 
 
@@ -382,38 +397,31 @@ handle_event(_Event, State) ->
 handle_info(secs3, State) ->
     {noreply, State#state{now = fmtnow()}};
 
-handle_info({notify,{yaws_hupped,_}}, State) ->
-    handle_info(minute10,State);
 %% once every 10 minutes, check log sizes
 handle_info(minute10, State) ->
-    L = lists:map(
-          fun(AL) ->
-                  wrap(AL, State)
-          end, State#state.alogs),
+    yaws_logger:rotate(State#state.log_wrap_size),
 
     Dir = State#state.dir,
     E = filename:join([Dir, "report.log"]),
     case file:read_file_info(E) of
-        {ok, FI} when  State#state.log_wrap_size > 0, 
-        FI#file_info.size > State#state.log_wrap_size, 
-        State#state.copy_errlog == true ->
+        {ok, FI} when  State#state.log_wrap_size > 0,
+                       FI#file_info.size > State#state.log_wrap_size,
+                       State#state.copy_errlog == true ->
             gen_event:call(error_logger, yaws_log_file_h, wrap, infinity);
         {error,enoent} ->
             gen_event:call(error_logger, yaws_log_file_h, reopen, infinity);
         _ ->
             ok
     end,
-    {noreply, State#state{alogs= L,
-                          auth_log = wrap(State#state.auth_log, State)}};
+    {noreply, State};
 handle_info({'EXIT', _, _}, State) ->
     {noreply, State}.
 
 
 
-wrap_p(AL, State) when is_record(AL, alog) ->
-    case file:read_file_info(AL#alog.filename) of
-        {ok, FI} when FI#file_info.size > State#state.log_wrap_size,
-        State#state.log_wrap_size > 0 ->
+wrap_p(Filename, LogWrapSize) ->
+    case file:read_file_info(Filename) of
+        {ok, FI} when FI#file_info.size > LogWrapSize, LogWrapSize > 0 ->
             true;
         {ok, _FI} ->
             false;
@@ -421,37 +429,10 @@ wrap_p(AL, State) when is_record(AL, alog) ->
             enoent;
         _ ->
             false
-    end;
-wrap_p(_,_) ->
-    false.
-
-
-
-wrap(AL, State) ->
-    case wrap_p(AL, State) of
-        true ->
-            file:close(AL#alog.fd),
-            Old = [AL#alog.filename, ".old"],
-            file:delete(Old),
-            file:rename(AL#alog.filename, Old),
-            {ok, Fd2} = file:open(AL#alog.filename, [write, raw]),
-            error_logger:info_msg("Wrap log ~p",[AL#alog.filename]), 
-            AL#alog{fd = Fd2};
-        false ->
-            AL;
-        enoent ->
-            %% Logfile disappeared, 
-            error_logger:format("Logfile ~p disappeared - we reopen it",
-                                [AL#alog.filename]),
-	    file:close(AL#alog.fd),
-            {ok, Fd2} = file:open(AL#alog.filename, [write, raw]),
-            AL#alog{fd = Fd2}
     end.
 
 
 
-
-
 %%----------------------------------------------------------------------
 %% Func: terminate/2
 %% Purpose: Shutdown the server
@@ -459,19 +440,43 @@ wrap(AL, State) ->
 %%----------------------------------------------------------------------
 terminate(_Reason, _State) ->
     gen_event:delete_handler(error_logger, yaws_log_file_h, normal),
+    yaws_logger:close_logs(),
     ok.
 
+
+%%----------------------------------------------------------------------
+%% Func: code_change/3
+%% Purpose: Handle upgrade
+%% Returns: new State data
+%%----------------------------------------------------------------------
+code_change(_OldVsn, Data, _Extra) ->
+    {ok, Data}.
+
 %%%----------------------------------------------------------------------
 %%% Internal functions
 %%%----------------------------------------------------------------------
+safe_decode_path({abs_path, Path}) ->
+    case catch yaws_api:url_decode(Path) of
+        {'EXIT', _} -> "/undecodable_path";
+        Val         -> Val
+    end;
+safe_decode_path(_) ->
+    "/undecodable_path".
 
 
-fmt_alog(Time, Ip, User, Req, Status,  Length, Referrer, UserAgent) ->
-    [fmt_ip(Ip), " - ", User, [$\s], Time, [$\s, $"], no_ctl(Req), [$",$\s], 
-     Status, [$\s], Length, [$\s,$"], Referrer, [$",$\s,$"], UserAgent, [$",$\n]].
+optional_header(Item) ->
+    case Item of
+        undefined -> "-";
+        Item -> Item
+    end.
+
+fmt_access_log(Time, Ip, User, Req, Status,  Length, Referrer, UserAgent) ->
+    [fmt_ip(Ip), " - ", User, [$\s], Time, [$\s, $"], no_ctl(Req), [$",$\s],
+     Status, [$\s], Length, [$\s,$"], Referrer, [$",$\s,$"], UserAgent,
+     [$",$\n]].
 
 
-%% Odd security advisory that only affects webservers where users are 
+%% Odd security advisory that only affects webservers where users are
 %% somehow allowed to upload files that later can be downloaded.
 
 no_ctl([H|T]) when H < 32 ->
@@ -491,16 +496,17 @@ fmt_ip(HostName) ->
 
 
 fmtnow() ->
-    {{Year, Month, Day}, {Hour, Min, Sec}} = 
+    {{Year, Month, Day}, {Hour, Min, Sec}} =
         calendar:now_to_local_time(now()),
     ["[",fill_zero(Day,2),"/",yaws:month(Month),"/",integer_to_list(Year),":",
-     fill_zero(Hour,2),":",fill_zero(Min,2),":",fill_zero(Sec,2)," ",zone(),"]"].
-    
+     fill_zero(Hour,2),":",fill_zero(Min,2),":",
+     fill_zero(Sec,2)," ",zone(),"]"].
+
 
 zone() ->
     Time = erlang:universaltime(),
     LocalTime = calendar:universal_time_to_local_time(Time),
-    DiffSecs = calendar:datetime_to_gregorian_seconds(LocalTime) - 
+    DiffSecs = calendar:datetime_to_gregorian_seconds(LocalTime) -
         calendar:datetime_to_gregorian_seconds(Time),
     zone(DiffSecs div 3600, (DiffSecs rem 3600) div 60).
 
